#coding:utf-8
import logging
import tornado.ioloop
import tornado.web
from msgserver.utils import get_class
from msgserver.utils.rabbitmq import RabbitBinding


DEFAULT_PARAMS = {
    'log_level': 'INFO',
    'socket_url': '/wsocket',
    'socket_port': '8001',
    'socket_router_name': 'msgserver.sockets.sock_js.SessionConnection',
    'session_engine': 'msgserver.sessions.db.AsyncSessions',
    'cookie_session_key': 'sessionid',
    'cache_backend': 'memcached://127.0.0.1:11211',
}


def run(rabbit_host, handlers_dict=None, user_params=None):
    params = DEFAULT_PARAMS.copy()
    if user_params:
        params.update(user_params)

    # получим класс соединения
    socket_router_name = params.get('socket_router_name')
    ConnectionRouter = get_class(socket_router_name)

    # получим класс получения django-сессий
    cookie_session_key = params.get('cookie_session_key')
    session_engine_class = get_class(params.get('session_engine'))
    session_engine = session_engine_class(params)

    # создадим роутер соединений и сконфигурируем соединение
    socket_url = params.get('socket_url')
    router = ConnectionRouter.create_router(socket_url, session_engine, cookie_session_key)
    # пропишем роутер в tornado-приложении
    app = tornado.web.Application(router.urls)
    ioloop = tornado.ioloop.IOLoop.instance()

    # подключим обработчики очередей RabbitMQ
    if handlers_dict:
        for queue, class_name in handlers_dict.items():
            if class_name:
                handler = get_class(class_name)
                logging.info('Set "%s" handler for rabbitmq "%s" queue.'%(class_name, queue))
                rabbit_bind = RabbitBinding(queue, rabbit_host, callback=handler(ConnectionRouter.clients).bind)
                ioloop.add_timeout(500, rabbit_bind.connect)

    # старт сервера
    socket_port = params.get('socket_port')
    app.listen(socket_port)
    ioloop.start()