#coding:utf-8
import base64
import functools
import pickle
from msgserver.utils.async_psycopg2 import Pool


class AsyncSessions(object):
    """
    Ассинхронный доступ к django-сессиям хранящимся в PostgreSql
    """
    def __init__(self, params):
        engine = params.get('database_engine')
        if not ('psycopg2' in engine):
            raise Exception('Incorrect database engine!')

        self.db_pool = Pool(
            1,
            20,
            10,
            **{
                'host': params.get('database_host'),
                'database': params.get('database_name'),
                'user': params.get('database_user'),
                'password': params.get('database_password'),
                'port': params.get('database_port'),
                'async': 1,
            })

    def get_user_id(self, session_id, callback):
        """
        Работает исключительно в связке с
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
            из-за raw sql к сессиям
        """
        sql = "select session_data from django_session where session_key='%s'" % session_id
        on_response_callback = functools.partial(self._on_db_response, callback)
        self.db_pool.execute(sql, callback=on_response_callback)

    def _on_db_response(self, callback, cursor):
        session_data = cursor.fetchone()
        if session_data:
            session = self._decode(session_data[0])
            callback(session.get('_auth_user_id'))

    def _decode(self, session_data):
        encoded_data = base64.decodestring(session_data)
        _, pickled = encoded_data.split(':', 1)
        return pickle.loads(pickled)